@@ -25,12 +25,11 @@ import logging
 import os
 import sys
 
-import xdg.BaseDirectory
-
 from logging.handlers import RotatingFileHandler
 
+from ubuntu_sso import xdg_base_directory
 
-LOGFOLDER = os.path.join(xdg.BaseDirectory.xdg_cache_home, 'sso')
+LOGFOLDER = os.path.join(xdg_base_directory.xdg_cache_home, 'sso')
 # create log folder if it doesn't exists
 if not os.path.exists(LOGFOLDER):
     os.makedirs(LOGFOLDER)
