@@ -28,6 +28,7 @@ local keyring.
 """
 
 import threading
+import warnings
 
 import dbus.service
 
@@ -45,7 +46,6 @@ from ubuntu_sso.main import (CredentialsManagementRoot, SSOLoginRoot,
 
 
 logger = setup_logging("ubuntu_sso.main")
-TIMEOUT_INTERVAL = 10000  # 10 seconds
 
 
 def blocking(f, app_name, result_cb, error_cb):
@@ -74,6 +74,8 @@ class SSOLogin(dbus.service.Object):
         dbus.service.Object.__init__(self, object_path=object_path,
                                      bus_name=bus_name)
         self.root = SSOLoginRoot(sso_login_processor_class, sso_service_class)
+        msg = 'Use ubuntu_sso.main.CredentialsManagement instead.'
+        warnings.warn(msg, DeprecationWarning)
 
     # generate_capcha signals
     @dbus.service.signal(DBUS_IFACE_USER_NAME, signature="ss")
@@ -92,7 +94,7 @@ class SSOLogin(dbus.service.Object):
                          in_signature='ss')
     def generate_captcha(self, app_name, filename):
         """Call the matching method in the processor."""
-        self.root.generate_captcha(app_name, filename, blocking,
+        self.root.generate_captcha(app_name, filename,
                                    self.CaptchaGenerated,
                                    self.CaptchaGenerationError)
 
@@ -115,7 +117,7 @@ class SSOLogin(dbus.service.Object):
                       captcha_id, captcha_solution):
         """Call the matching method in the processor."""
         self.root.register_user(app_name, email, password, name, captcha_id,
-                                captcha_solution, blocking,
+                                captcha_solution,
                                 self.UserRegistered,
                                 self.UserRegistrationError)
 
@@ -142,7 +144,7 @@ class SSOLogin(dbus.service.Object):
                          in_signature='sss')
     def login(self, app_name, email, password):
         """Call the matching method in the processor."""
-        self.root.login(app_name, email, password, blocking, self.LoggedIn,
+        self.root.login(app_name, email, password, self.LoggedIn,
                         self.LoginError, self.UserNotValidated)
 
     # validate_email signals
@@ -163,7 +165,7 @@ class SSOLogin(dbus.service.Object):
     def validate_email(self, app_name, email, password, email_token):
         """Call the matching method in the processor."""
         self.root.validate_email(app_name, email, password, email_token,
-                                 blocking, self.EmailValidated,
+                                 self.EmailValidated,
                                  self.EmailValidationError)
 
     # request_password_reset_token signals
@@ -183,7 +185,7 @@ class SSOLogin(dbus.service.Object):
                          in_signature='ss')
     def request_password_reset_token(self, app_name, email):
         """Call the matching method in the processor."""
-        self.root.request_password_reset_token(app_name, email, blocking,
+        self.root.request_password_reset_token(app_name, email,
                                                self.PasswordResetTokenSent,
                                                self.PasswordResetError)
 
@@ -205,12 +207,16 @@ class SSOLogin(dbus.service.Object):
     def set_new_password(self, app_name, email, token, new_password):
         """Call the matching method in the processor."""
         self.root.set_new_password(app_name, email, token, new_password,
-                                   blocking, self.PasswordChanged,
+                                   self.PasswordChanged,
                                    self.PasswordChangeError)
 
 
 class SSOCredentials(dbus.service.Object):
-    """DBus object that gets credentials, and login/registers if needed."""
+    """DBus object that gets credentials, and login/registers if needed.
+
+    This class is Deprecated. DO NOT USE, use CredentialsManagement instead.
+
+    """
 
     # Operator not preceded by a space (fails with dbus decorators)
     # pylint: disable=C0322
@@ -218,6 +224,9 @@ class SSOCredentials(dbus.service.Object):
     def __init__(self, *args, **kwargs):
         dbus.service.Object.__init__(self, *args, **kwargs)
         self.root = SSOCredentialsRoot()
+        warnings.warn('%r DBus object is deprecated, please use %r instead.' %
+                      (DBUS_IFACE_CRED_NAME, DBUS_CREDENTIALS_IFACE),
+                      DeprecationWarning)
 
     def _process_error(self, app_name, error_dict):
         """Process the 'error_dict' and emit CredentialsError."""
@@ -266,11 +275,12 @@ class SSOCredentials(dbus.service.Object):
 
         """
         self.root.login_or_register_to_get_credentials(app_name,
-                                                     terms_and_conditions_url,
-                                                     help_text, window_id,
-                                                     self.CredentialsFound,
-                                                     self._process_error,
-                                                     self.AuthorizationDenied)
+                                               terms_and_conditions_url,
+                                               help_text, window_id,
+                                               self.CredentialsFound,
+                                               self._process_error,
+                                               self.AuthorizationDenied,
+                                               ui_module='ubuntu_sso.gtk.gui')
 
     @dbus.service.method(dbus_interface=DBUS_IFACE_CRED_NAME,
                          in_signature="ssx", out_signature="")
@@ -287,7 +297,8 @@ class SSOCredentials(dbus.service.Object):
         self.root.login_to_get_credentials(app_name, help_text, window_id,
                                            self.CredentialsFound,
                                            self._process_error,
-                                           self.AuthorizationDenied)
+                                           self.AuthorizationDenied,
+                                           ui_module='ubuntu_sso.gtk.gui')
 
     @dbus.service.method(dbus_interface=DBUS_IFACE_CRED_NAME,
                          in_signature='s', out_signature='',
@@ -328,10 +339,8 @@ class CredentialsManagement(dbus.service.Object):
 
     def __init__(self, timeout_func, shutdown_func, *args, **kwargs):
         super(CredentialsManagement, self).__init__(*args, **kwargs)
-        self._ref_count = 0
-        self.timeout_func = timeout_func
-        self.shutdown_func = shutdown_func
-        self.root = CredentialsManagementRoot(self.CredentialsFound,
+        self.root = CredentialsManagementRoot(timeout_func, shutdown_func,
+                                              self.CredentialsFound,
                                               self.CredentialsError,
                                               self.AuthorizationDenied)
 
@@ -342,74 +351,50 @@ class CredentialsManagement(dbus.service.Object):
         """Process the 'failure' and emit CredentialsError."""
         self.CredentialsError(app_name, except_to_errdict(failure.value))
 
-    def _get_ref_count(self):
-        """Get value of ref_count."""
-        return self._ref_count
-
-    def _set_ref_count(self, new_value):
-        """Set a new value to ref_count."""
-        logger.debug('ref_count is %r, changing value to %r.',
-                     self._ref_count, new_value)
-        if new_value < 0:
-            self._ref_count = 0
-            msg = 'Attempting to decrease ref_count to a negative value (%r).'
-            logger.warning(msg, new_value)
-        else:
-            self._ref_count = new_value
-
-        if self._ref_count == 0:
-            logger.debug('Setting up timer with %r (%r, %r).',
-                         self.timeout_func, TIMEOUT_INTERVAL, self.shutdown)
-            self.timeout_func(TIMEOUT_INTERVAL, self.shutdown)
-
-    ref_count = property(fget=_get_ref_count, fset=_set_ref_count)
-
     def shutdown(self):
         """If no ongoing requests, call self.shutdown_func."""
-        logger.debug('shutdown!, ref_count is %r.', self._ref_count)
-        if self._ref_count == 0:
-            logger.info('Shutting down, calling %r.', self.shutdown_func)
-            self.shutdown_func()
+        logger.debug('shutdown!, ref_count is %r.', self.root.ref_count)
+        self.root.shutdown()
 
     @dbus.service.signal(DBUS_CREDENTIALS_IFACE, signature='s')
     def AuthorizationDenied(self, app_name):
         """Signal thrown when the user denies the authorization."""
-        self.ref_count -= 1
+        self.root.ref_count -= 1
         logger.info('%s: emitting AuthorizationDenied with app_name "%s".',
                     self.__class__.__name__, app_name)
 
     @dbus.service.signal(DBUS_CREDENTIALS_IFACE, signature='sa{ss}')
     def CredentialsFound(self, app_name, credentials):
         """Signal thrown when the credentials are found."""
-        self.ref_count -= 1
+        self.root.ref_count -= 1
         logger.info('%s: emitting CredentialsFound with app_name "%s".',
                     self.__class__.__name__, app_name)
 
     @dbus.service.signal(DBUS_CREDENTIALS_IFACE, signature='s')
     def CredentialsNotFound(self, app_name):
         """Signal thrown when the credentials are not found."""
-        self.ref_count -= 1
+        self.root.ref_count -= 1
         logger.info('%s: emitting CredentialsNotFound with app_name "%s".',
                     self.__class__.__name__, app_name)
 
     @dbus.service.signal(DBUS_CREDENTIALS_IFACE, signature='s')
     def CredentialsCleared(self, app_name):
         """Signal thrown when the credentials were cleared."""
-        self.ref_count -= 1
+        self.root.ref_count -= 1
         logger.info('%s: emitting CredentialsCleared with app_name "%s".',
                     self.__class__.__name__, app_name)
 
     @dbus.service.signal(DBUS_CREDENTIALS_IFACE, signature='s')
     def CredentialsStored(self, app_name):
         """Signal thrown when the credentials were cleared."""
-        self.ref_count -= 1
+        self.root.ref_count -= 1
         logger.info('%s: emitting CredentialsStored with app_name "%s".',
                     self.__class__.__name__, app_name)
 
     @dbus.service.signal(DBUS_CREDENTIALS_IFACE, signature='sa{ss}')
     def CredentialsError(self, app_name, error_dict):
         """Signal thrown when there is a problem getting the credentials."""
-        self.ref_count -= 1
+        self.root.ref_count -= 1
         logger.error('%s: emitting CredentialsError with app_name "%s" and '
                      'error_dict %r.', self.__class__.__name__, app_name,
                      error_dict)
@@ -425,7 +410,6 @@ class CredentialsManagement(dbus.service.Object):
         - 'args' is a dictionary, currently not used.
 
         """
-        self.ref_count += 1
 
         def success_cb(credentials):
             """Find credentials and notify using signals."""
@@ -448,7 +432,6 @@ class CredentialsManagement(dbus.service.Object):
         - 'args' is a dictionary, currently not used.
 
         """
-        self.ref_count += 1
         self.root.clear_credentials(app_name, args,
                                 lambda _: self.CredentialsCleared(app_name),
                                 self._process_failure)
@@ -466,7 +449,6 @@ class CredentialsManagement(dbus.service.Object):
         'consumer_secret'.
 
         """
-        self.ref_count += 1
         self.root.store_credentials(app_name, args,
                                 lambda _: self.CredentialsStored(app_name),
                                 self._process_failure)
@@ -475,12 +457,22 @@ class CredentialsManagement(dbus.service.Object):
                          in_signature='sa{ss}', out_signature='')
     def register(self, app_name, args):
         """Get credentials if found else prompt GUI to register."""
-        self.ref_count += 1
         self.root.register(app_name, args)
 
     @dbus.service.method(dbus_interface=DBUS_CREDENTIALS_IFACE,
                          in_signature='sa{ss}', out_signature='')
     def login(self, app_name, args):
         """Get credentials if found else prompt GUI to login."""
-        self.ref_count += 1
         self.root.login(app_name, args)
+
+    @dbus.service.method(dbus_interface=DBUS_CREDENTIALS_IFACE,
+                         in_signature='sa{ss}', out_signature='')
+    def login_email_password(self, app_name, args):
+        """Get credentials if found, else login using email and password.
+
+        - 'args' should contain at least the follwing keys: 'email' and
+        'password'. Those will be used to issue a new SSO token, which will be
+        returned trough the CredentialsFound signal.
+
+        """
+        self.root.login_email_password(app_name, args)
