@@ -23,7 +23,7 @@
 import logging
 import os
 
-from mocker import Mocker, MockerTestCase, ARGS, KWARGS
+from mocker import Mocker, MockerTestCase, ARGS, KWARGS, MATCH
 from twisted.internet import defer
 from twisted.internet.defer import Deferred, inlineCallbacks
 from twisted.trial.unittest import TestCase
@@ -36,8 +36,8 @@ import ubuntu_sso.main.linux
 from ubuntu_sso import DBUS_CREDENTIALS_IFACE
 from ubuntu_sso.keyring import U1_APP_NAME
 from ubuntu_sso.main import (U1_PING_URL, except_to_errdict,
-    CredentialsManagement, SSOCredentials, SSOLogin)
-from ubuntu_sso.main.linux import TIMEOUT_INTERVAL, blocking
+    CredentialsManagement, SSOCredentials, SSOLogin, TIMEOUT_INTERVAL)
+from ubuntu_sso.main.linux import blocking
 from ubuntu_sso.main import (HELP_TEXT_KEY, PING_URL_KEY,
     TC_URL_KEY, UI_CLASS_KEY, UI_MODULE_KEY, WINDOW_ID_KEY,
     SUCCESS_CB_KEY, ERROR_CB_KEY, DENIAL_CB_KEY)
@@ -144,7 +144,7 @@ class SsoDbusTestCase(TestCase):
         expected_result = "expected result"
         self.create_mock_processor().generate_captcha(filename)
         self.mocker.result(expected_result)
-        self.patch(ubuntu_sso.main.linux, "blocking", fake_err_blocking)
+        self.patch(ubuntu_sso.main, "thread_execute", fake_err_blocking)
         self.mocker.replay()
 
         def verify(app_name, errdict):
@@ -191,7 +191,7 @@ class SsoDbusTestCase(TestCase):
         self.create_mock_processor().register_user(EMAIL, PASSWORD, NAME,
             CAPTCHA_ID, CAPTCHA_SOLUTION)
         self.mocker.result(expected_result)
-        self.patch(ubuntu_sso.main.linux, "blocking", fake_err_blocking)
+        self.patch(ubuntu_sso.main, "thread_execute", fake_err_blocking)
         self.mocker.replay()
 
         def verify(app_name, errdict):
@@ -377,7 +377,7 @@ class SsoDbusTestCase(TestCase):
         d = Deferred()
         processor = self.create_mock_processor()
         processor.request_password_reset_token(EMAIL)
-        self.patch(ubuntu_sso.main.linux, "blocking", fake_ok_blocking)
+        self.patch(ubuntu_sso.main, "thread_execute", fake_ok_blocking)
         self.mocker.result(EMAIL)
         self.mocker.replay()
 
@@ -400,7 +400,7 @@ class SsoDbusTestCase(TestCase):
 
         self.create_mock_processor().request_password_reset_token(EMAIL)
         self.mocker.result(EMAIL)
-        self.patch(ubuntu_sso.main.linux, "blocking", fake_err_blocking)
+        self.patch(ubuntu_sso.main, "thread_execute", fake_err_blocking)
         self.mocker.replay()
 
         def verify(app_name, errdict):
@@ -446,7 +446,7 @@ class SsoDbusTestCase(TestCase):
         self.create_mock_processor().set_new_password(EMAIL, EMAIL_TOKEN,
                                                       PASSWORD)
         self.mocker.result(expected_result)
-        self.patch(ubuntu_sso.main.linux, "blocking", fake_err_blocking)
+        self.patch(ubuntu_sso.main, "thread_execute", fake_err_blocking)
         self.mocker.replay()
 
         def verify(app_name, errdict):
@@ -623,6 +623,7 @@ class ApplicationCredentialsGUITestCase(TestCase, MockerTestCase):
 
     def test_login_or_register(self):
         """login_or_register is correct."""
+        self.args[UI_MODULE_KEY] = MATCH(lambda x: isinstance(x, str))
         mock_class = self.mocker.replace("ubuntu_sso.credentials.Credentials")
         mock_class(app_name=self.app_name, **self.args)
         creds_obj = self.mocker.mock()
@@ -637,6 +638,7 @@ class ApplicationCredentialsGUITestCase(TestCase, MockerTestCase):
     def test_login_only(self):
         """login_or_register is correct."""
         self.args[TC_URL_KEY] = None
+        self.args[UI_MODULE_KEY] = MATCH(lambda x: isinstance(x, str))
         mock_class = self.mocker.replace("ubuntu_sso.credentials.Credentials")
         mock_class(app_name=self.app_name, **self.args)
         creds_obj = self.mocker.mock()
@@ -770,7 +772,7 @@ class CredentialsManagementRefCountingTestCase(CredentialsManagementTestCase):
 
     def test_ref_counting(self):
         """Ref counting is in place."""
-        self.assertEqual(self.client.ref_count, 0)
+        self.assertEqual(self.client.root.ref_count, 0)
 
     def test_find_credentials(self):
         """Keep proper track of on going requests."""
@@ -778,7 +780,7 @@ class CredentialsManagementRefCountingTestCase(CredentialsManagementTestCase):
 
         def verify(*args):
             """Make the check."""
-            self.assertEqual(self.client.ref_count, 1)
+            self.assertEqual(self.client.root.ref_count, 1)
             d.callback(True)
 
         self.patch(self.client, 'CredentialsFound', verify)
@@ -792,7 +794,7 @@ class CredentialsManagementRefCountingTestCase(CredentialsManagementTestCase):
 
         def verify(*args):
             """Make the check."""
-            self.assertEqual(self.client.ref_count, 1)
+            self.assertEqual(self.client.root.ref_count, 1)
             d.callback(True)
 
         self.patch(self.client, 'CredentialsCleared', verify)
@@ -806,7 +808,7 @@ class CredentialsManagementRefCountingTestCase(CredentialsManagementTestCase):
 
         def verify(*args):
             """Make the check."""
-            self.assertEqual(self.client.ref_count, 1)
+            self.assertEqual(self.client.root.ref_count, 1)
             d.callback(True)
 
         self.patch(self.client, 'CredentialsStored', verify)
@@ -818,13 +820,13 @@ class CredentialsManagementRefCountingTestCase(CredentialsManagementTestCase):
         """Keep proper track of on going requests."""
         self.client.register(APP_NAME, self.args)
 
-        self.assertEqual(self.client.ref_count, 1)
+        self.assertEqual(self.client.root.ref_count, 1)
 
     def test_login(self):
         """Keep proper track of on going requests."""
         self.client.login(APP_NAME, self.args)
 
-        self.assertEqual(self.client.ref_count, 1)
+        self.assertEqual(self.client.root.ref_count, 1)
 
     def test_several_requests(self):
         """Requests can be nested."""
@@ -834,112 +836,112 @@ class CredentialsManagementRefCountingTestCase(CredentialsManagementTestCase):
         self.client.register(APP_NAME, self.args)
         self.client.register(APP_NAME, self.args)
 
-        self.assertEqual(self.client.ref_count, 5)
+        self.assertEqual(self.client.root.ref_count, 5)
 
     def test_credentials_found(self):
         """Ref counter is decreased when a signal is sent."""
-        self.client.ref_count = 3
+        self.client.root.ref_count = 3
         self.client.CredentialsFound(APP_NAME, TOKEN)
 
-        self.assertEqual(self.client.ref_count, 2)
+        self.assertEqual(self.client.root.ref_count, 2)
 
     def test_credentials_not_found(self):
         """Ref counter is decreased when a signal is sent."""
-        self.client.ref_count = 3
+        self.client.root.ref_count = 3
         self.client.CredentialsNotFound(APP_NAME)
 
-        self.assertEqual(self.client.ref_count, 2)
+        self.assertEqual(self.client.root.ref_count, 2)
 
     def test_credentials_cleared(self):
         """Ref counter is decreased when a signal is sent."""
-        self.client.ref_count = 3
+        self.client.root.ref_count = 3
         self.client.CredentialsCleared(APP_NAME)
 
-        self.assertEqual(self.client.ref_count, 2)
+        self.assertEqual(self.client.root.ref_count, 2)
 
     def test_credentials_stored(self):
         """Ref counter is decreased when a signal is sent."""
-        self.client.ref_count = 3
+        self.client.root.ref_count = 3
         self.client.CredentialsStored(APP_NAME)
 
-        self.assertEqual(self.client.ref_count, 2)
+        self.assertEqual(self.client.root.ref_count, 2)
 
     def test_credentials_error(self):
         """Ref counter is decreased when a signal is sent."""
-        self.client.ref_count = 3
+        self.client.root.ref_count = 3
         self.client.CredentialsError(APP_NAME, {'error_type': 'test'})
 
-        self.assertEqual(self.client.ref_count, 2)
+        self.assertEqual(self.client.root.ref_count, 2)
 
     def test_authorization_denied(self):
         """Ref counter is decreased when a signal is sent."""
-        self.client.ref_count = 3
+        self.client.root.ref_count = 3
         self.client.AuthorizationDenied(APP_NAME)
 
-        self.assertEqual(self.client.ref_count, 2)
+        self.assertEqual(self.client.root.ref_count, 2)
 
     def test_credentials_found_when_ref_count_is_not_positive(self):
         """Ref counter is decreased when a signal is sent."""
-        self.client._ref_count = -3
+        self.client.root._ref_count = -3
         self.client.CredentialsFound(APP_NAME, TOKEN)
 
-        self.assertEqual(self.client.ref_count, 0)
+        self.assertEqual(self.client.root.ref_count, 0)
         msg = 'Attempting to decrease ref_count to a negative value (-4).'
         self.assertTrue(self.memento.check_warning(msg))
 
     def test_credentials_not_found_when_ref_count_is_not_positive(self):
         """Ref counter is decreased when a signal is sent."""
-        self.client._ref_count = -3
+        self.client.root._ref_count = -3
         self.client.CredentialsNotFound(APP_NAME)
 
-        self.assertEqual(self.client.ref_count, 0)
+        self.assertEqual(self.client.root.ref_count, 0)
         msg = 'Attempting to decrease ref_count to a negative value (-4).'
         self.assertTrue(self.memento.check_warning(msg))
 
     def test_credentials_cleared_when_ref_count_is_not_positive(self):
         """Ref counter is decreased when a signal is sent."""
-        self.client._ref_count = -3
+        self.client.root._ref_count = -3
         self.client.CredentialsCleared(APP_NAME)
 
-        self.assertEqual(self.client.ref_count, 0)
+        self.assertEqual(self.client.root.ref_count, 0)
         msg = 'Attempting to decrease ref_count to a negative value (-4).'
         self.assertTrue(self.memento.check_warning(msg))
 
     def test_credentials_stored_when_ref_count_is_not_positive(self):
         """Ref counter is decreased when a signal is sent."""
-        self.client._ref_count = -3
+        self.client.root._ref_count = -3
         self.client.CredentialsStored(APP_NAME)
 
-        self.assertEqual(self.client.ref_count, 0)
+        self.assertEqual(self.client.root.ref_count, 0)
         msg = 'Attempting to decrease ref_count to a negative value (-4).'
         self.assertTrue(self.memento.check_warning(msg))
 
     def test_credentials_error_when_ref_count_is_not_positive(self):
         """Ref counter is decreased when a signal is sent."""
-        self.client._ref_count = -3
+        self.client.root._ref_count = -3
         self.client.CredentialsError(APP_NAME, {'error_type': 'test'})
 
-        self.assertEqual(self.client.ref_count, 0)
+        self.assertEqual(self.client.root.ref_count, 0)
         msg = 'Attempting to decrease ref_count to a negative value (-4).'
         self.assertTrue(self.memento.check_warning(msg))
 
     def test_autorization_denied_when_ref_count_is_not_positive(self):
         """Ref counter is decreased when a signal is sent."""
-        self.client._ref_count = -3
+        self.client.root._ref_count = -3
         self.client.AuthorizationDenied(APP_NAME)
 
-        self.assertEqual(self.client.ref_count, 0)
+        self.assertEqual(self.client.root.ref_count, 0)
         msg = 'Attempting to decrease ref_count to a negative value (-4).'
         self.assertTrue(self.memento.check_warning(msg))
 
     def test_on_zero_ref_count_shutdown(self):
         """When ref count reaches 0, queue shutdown op."""
-        self.client.timeout_func = self._set_called
+        self.client.root.timeout_func = self._set_called
         self.client.login(APP_NAME, self.args)
         self.client.CredentialsFound(APP_NAME, TOKEN)
 
         self.assertEqual(self._called,
-                         ((TIMEOUT_INTERVAL, self.client.shutdown), {}))
+                         ((TIMEOUT_INTERVAL, self.client.root.shutdown), {}))
 
     def test_on_non_zero_ref_count_do_not_shutdown(self):
         """If ref count is not 0, do not queue shutdown op."""
@@ -954,7 +956,7 @@ class CredentialsManagementRefCountingTestCase(CredentialsManagementTestCase):
         def fake_timeout_func(interval, func):
             """Start a new request when the timer is started."""
             self.client.register(APP_NAME, self.args)
-            assert self.client.ref_count > 0
+            assert self.client.root.ref_count > 0
             func()
 
         self.client.timeout_func = fake_timeout_func
@@ -971,11 +973,11 @@ class CredentialsManagementRefCountingTestCase(CredentialsManagementTestCase):
 
         def fake_timeout_func(interval, func):
             """Start a new request when the timer is started."""
-            assert self.client.ref_count == 0
+            assert self.client.root.ref_count == 0
             func()
 
-        self.client.timeout_func = fake_timeout_func
-        self.client.shutdown_func = self._set_called
+        self.client.root.timeout_func = fake_timeout_func
+        self.client.root.shutdown_func = self._set_called
 
         self.client.login(APP_NAME, self.args)
         self.client.CredentialsFound(APP_NAME, TOKEN)
@@ -1220,6 +1222,17 @@ class CredentialsManagementOpsTestCase(CredentialsManagementTestCase):
         self.client.login(APP_NAME, self.args)
         self.assert_dbus_method_correct(self.client.login)
 
+    def test_login_email_password(self):
+        """The login_email_password is correct."""
+        self.create_mock_backend().login_email_password(email=EMAIL,
+                                                        password=PASSWORD)
+        self.mocker.replay()
+
+        self.args['email'] = EMAIL
+        self.args['password'] = PASSWORD
+        self.client.login_email_password(APP_NAME, self.args)
+        self.assert_dbus_method_correct(self.client.login_email_password)
+
 
 class CredentialsManagementParamsTestCase(CredentialsManagementOpsTestCase):
     """Tests for the CredentialsManagement extra parameters handling."""
