@@ -22,30 +22,30 @@ from twisted.trial import unittest
 
 from ubuntu_sso.keyring import get_token_name
 
-APP_NAME = 'The Super App!'
-CAPTCHA_ID = 'test'
+APP_NAME = u'The Super App!'
+CAPTCHA_ID = u'test'
 CAPTCHA_PATH = os.path.abspath(os.path.join(os.curdir, 'ubuntu_sso', 'tests',
                                             'files', 'captcha.png'))
-CAPTCHA_SOLUTION = 'william Byrd'
-EMAIL = 'test@example.com'
-EMAIL_TOKEN = 'B2Pgtf'
+CAPTCHA_SOLUTION = u'william Byrd'
+EMAIL = u'test@example.com'
+EMAIL_TOKEN = u'B2Pgtf'
 GTK_GUI_CLASS = 'UbuntuSSOClientGUI'
 GTK_GUI_MODULE = 'ubuntu_sso.gtk.gui'
 HELP_TEXT = """Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nam sed
 lorem nibh. Suspendisse gravida nulla non nunc suscipit pulvinar tempus ut
 augue. Morbi consequat, ligula a elementum pretium, dolor nulla tempus metus,
 sed viverra nisi risus non velit."""
-NAME = 'Juanito Pérez'
-PASSWORD = 'h3lloWorld'
-PING_URL = 'http://localhost/ping-me/'
-RESET_PASSWORD_TOKEN = '8G5Wtq'
+NAME = u'Juanito Pérez'
+PASSWORD = u'h3lloWorld'
+PING_URL = u'http://localhost/ping-me/'
+RESET_PASSWORD_TOKEN = u'8G5Wtq'
 TOKEN = {u'consumer_key': u'xQ7xDAz',
          u'consumer_secret': u'KzCJWCTNbbntwfyCKKjomJDzlgqxLy',
          u'token_name': u'test',
          u'token': u'GkInOfSMGwTXAUoVQwLUoPxElEEUdhsLVNTPhxHJDUIeHCPNEo',
          u'token_secret': u'qFYImEtlczPbsCnYyuwLoPDlPEnvNcIktZphPQklAWrvyfFMV'}
 TOKEN_NAME = get_token_name(APP_NAME)
-TC_URL = 'http://localhost/'
+TC_URL = u'http://localhost/'
 WINDOW_ID = 5
 
 
