@@ -19,19 +19,20 @@
 """Tests for the Credentials module."""
 
 import logging
-import urllib
+import os
+import urllib2
 
 from twisted.internet import defer
-from twisted.internet.defer import inlineCallbacks
 from twisted.trial.unittest import TestCase, FailTest
 from ubuntuone.devtools.handlers import MementoHandler
 
 from ubuntu_sso import credentials
+import ubuntu_sso.main
 from ubuntu_sso.credentials import (APP_NAME_KEY, HELP_TEXT_KEY, NO_OP,
     PING_URL_KEY, TC_URL_KEY, UI_CLASS_KEY, UI_MODULE_KEY, WINDOW_ID_KEY,
     ERROR_KEY, ERROR_DETAIL_KEY)
 from ubuntu_sso.tests import (APP_NAME, EMAIL, GTK_GUI_CLASS, GTK_GUI_MODULE,
-    HELP_TEXT, PING_URL, TC_URL, TOKEN, WINDOW_ID)
+    HELP_TEXT, PASSWORD, PING_URL, TC_URL, TOKEN, WINDOW_ID)
 
 
 # Access to a protected member of a client class
@@ -40,6 +41,9 @@ from ubuntu_sso.tests import (APP_NAME, EMAIL, GTK_GUI_CLASS, GTK_GUI_MODULE,
 # Attribute defined outside __init__
 # pylint: disable=W0201
 
+# Instance of 'class' has no 'x' member (but some types could not be inferred)
+# pylint: disable=E1103
+
 
 KWARGS = {
     APP_NAME_KEY: APP_NAME,
@@ -63,11 +67,13 @@ class SampleMiscException(Exception):
     """An error to be used while testing."""
 
 
-class FailingClientGUI(object):
-    """Fake a failing SSO GUI."""
+class FailingClient(object):
+    """Fake a failing client."""
+
+    err_msg = 'A failing class.'
 
     def __init__(self, *args, **kwargs):
-        raise SampleMiscException('A failing GUI class.')
+        raise SampleMiscException(self.err_msg)
 
 
 class FakedClientGUI(object):
@@ -81,6 +87,18 @@ class FakedClientGUI(object):
         self.user_cancellation_callback = None
 
 
+class FakedSSOLoginRoot(object):
+    """Fake a SSOLoginRoot."""
+
+    args = []
+    kwargs = {}
+
+    def login(self, *args, **kwargs):
+        """Fake login."""
+        self.args = args
+        self.kwargs = kwargs
+
+
 class BasicTestCase(TestCase):
     """Test case with a helper tracker."""
 
@@ -112,10 +130,6 @@ class CredentialsTestCase(BasicTestCase):
                                            denial_cb=self.denial,
                                            **KWARGS)
 
-    def tearDown(self):
-        """Clean up."""
-        super(CredentialsTestCase, self).tearDown()
-
     def success(self, *args, **kwargs):
         """To be called on success."""
         self._set_called('success', *args, **kwargs)
@@ -231,7 +245,7 @@ class CredentialsCallbacksTestCase(CredentialsTestCase):
 class CredentialsLoginSuccessTestCase(CredentialsTestCase):
     """Test suite for the Credentials login success callback."""
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_cred_not_found(self):
         """On auth success, if cred not found, self.error_cb is called."""
         self.patch(credentials.Keyring, 'get_credentials',
@@ -244,7 +258,7 @@ class CredentialsLoginSuccessTestCase(CredentialsTestCase):
                                     detailed_error=AssertionError(msg))
         self.assertEqual(result, None)
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_cred_error(self):
         """On auth success, if cred failed, self.error_cb is called."""
         expected_error = SampleMiscException()
@@ -258,7 +272,7 @@ class CredentialsLoginSuccessTestCase(CredentialsTestCase):
         self.assertEqual(result, None)
         self.assertTrue(self.memento.check_exception(SampleMiscException))
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_ping_success(self):
         """Auth success + cred found + ping success, success_cb is called."""
         self.patch(credentials.Keyring, 'get_credentials',
@@ -270,7 +284,7 @@ class CredentialsLoginSuccessTestCase(CredentialsTestCase):
         self.assertEqual(self._called, (('success', APP_NAME, TOKEN), {}))
         self.assertEqual(result, 0)
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_ping_error(self):
         """Auth success + cred found + ping error, error_cb is called.
 
@@ -298,7 +312,7 @@ class CredentialsLoginSuccessTestCase(CredentialsTestCase):
         # exception logged
         self.assertTrue(self.memento.check_exception(FailTest, error))
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_pings_url(self):
         """On auth success, self.ping_url is opened."""
         self.patch(credentials.Keyring, 'get_credentials',
@@ -314,7 +328,7 @@ class CredentialsLoginSuccessTestCase(CredentialsTestCase):
 
         self.assertEqual(self._url_pinged, ((APP_NAME, EMAIL, TOKEN), {}))
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_no_ping_url_is_success(self):
         """Auth success + cred found + no ping url, success_cb is called.
 
@@ -352,15 +366,15 @@ class PingUrlTestCase(CredentialsTestCase):
         def faked_urlopen(request):
             """Fake urlopener."""
             self._request = request
-            response = urllib.addinfourl(fp=open('/dev/null'),
-                                         headers=request.headers,
-                                         url=request.get_full_url(),
-                                         code=200)
+            response = urllib2.addinfourl(fp=open(os.path.devnull),
+                                          headers=request.headers,
+                                          url=request.get_full_url(),
+                                          code=200)
             return response
 
         self.patch(credentials.urllib2, 'urlopen', faked_urlopen)
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_ping_url_if_url_is_none(self):
         """self.ping_url is opened."""
         self.patch(credentials.urllib2, 'urlopen', self.fail)
@@ -369,7 +383,7 @@ class PingUrlTestCase(CredentialsTestCase):
                                  credentials=TOKEN)
         # no failure
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_ping_url(self):
         """On auth success, self.ping_url is opened."""
         yield self.obj._ping_url(app_name=APP_NAME, email=EMAIL,
@@ -379,23 +393,23 @@ class PingUrlTestCase(CredentialsTestCase):
         self.assertEqual(self._request.get_full_url(),
                          self.obj.ping_url + EMAIL)
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_request_is_signed_with_credentials(self):
         """The http request to self.ping_url is signed with the credentials."""
+
+        def fake_it(*a, **kw):
+            """Fake oauth_headers."""
+            self._set_called(*a, **kw)
+            return {}
+
+        self.patch(credentials.utils, 'oauth_headers', fake_it)
         result = yield self.obj._ping_url(APP_NAME, EMAIL, TOKEN)
-        headers = self._request.headers
 
-        self.assertIn('Authorization', headers)
-        oauth_stuff = headers['Authorization']
+        self.assertEqual(self._called,
+                         ((self.obj.ping_url + EMAIL, TOKEN), {}))
+        self.assertEqual(result.code, 200)
 
-        expected = 'oauth_consumer_key="xQ7xDAz", ' \
-            'oauth_signature_method="HMAC-SHA1", oauth_version="1.0", ' \
-            'oauth_token="GkInOfSMGwTXAUoVQwLUoPxElEEUdhsLVNTPhxHJDUIeHCPNEo' \
-            '", oauth_signature="'
-        self.assertIn(expected, oauth_stuff)
-        self.assertEqual(result, 200)
-
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_ping_url_error(self):
         """Exception is handled if ping fails."""
         error = 'Blu'
@@ -407,12 +421,52 @@ class PingUrlTestCase(CredentialsTestCase):
         self.assert_error_cb_called(msg=msg, detailed_error=FailTest(error))
         self.assertTrue(self.memento.check_exception(FailTest, error))
 
+    @defer.inlineCallbacks
+    def test_ping_url_formatting(self):
+        """The email is added as the first formatting argument."""
+        self.obj.ping_url = u'http://example.com/{email}/something/else'
+        result = yield self.obj._ping_url(app_name=APP_NAME, email=EMAIL,
+                                          credentials=TOKEN)
+
+        expected = self.obj.ping_url.format(email=EMAIL)
+        self.assertEqual(expected, result.url)
+
+    @defer.inlineCallbacks
+    def test_ping_url_formatting_with_query_params(self):
+        """The email is added as the first formatting argument."""
+        self.obj.ping_url = u'http://example.com/{email}?something=else'
+        result = yield self.obj._ping_url(app_name=APP_NAME, email=EMAIL,
+                                          credentials=TOKEN)
+
+        expected = self.obj.ping_url.format(email=EMAIL)
+        self.assertEqual(expected, result.url)
+
+    @defer.inlineCallbacks
+    def test_ping_url_formatting_no_email_kwarg(self):
+        """The email is added as the first formatting argument."""
+        self.obj.ping_url = u'http://example.com/{0}/yadda/?something=else'
+        result = yield self.obj._ping_url(app_name=APP_NAME, email=EMAIL,
+                                          credentials=TOKEN)
+
+        expected = self.obj.ping_url.format(EMAIL)
+        self.assertEqual(expected, result.url)
+
+    @defer.inlineCallbacks
+    def test_ping_url_formatting_no_format(self):
+        """The email is appended if formatting could not be accomplished."""
+        self.obj.ping_url = u'http://example.com/yadda/'
+        result = yield self.obj._ping_url(app_name=APP_NAME, email=EMAIL,
+                                          credentials=TOKEN)
+
+        expected = self.obj.ping_url + EMAIL
+        self.assertEqual(expected, result.url)
+
 
 class FindCredentialsTestCase(CredentialsTestCase):
     """Test suite for the find_credentials method."""
     timeout = 5
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_find_credentials(self):
         """A deferred with credentials is returned when found."""
         self.patch(credentials.Keyring, 'get_credentials',
@@ -421,7 +475,7 @@ class FindCredentialsTestCase(CredentialsTestCase):
         token = yield self.obj.find_credentials()
         self.assertEqual(token, TOKEN)
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_credentials_not_found(self):
         """find_credentials returns {} when no credentials are found."""
         self.patch(credentials.Keyring, 'get_credentials',
@@ -430,7 +484,7 @@ class FindCredentialsTestCase(CredentialsTestCase):
         token = yield self.obj.find_credentials()
         self.assertEqual(token, {})
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_keyring_failure(self):
         """Failures from the keyring are handled."""
         expected_error = SampleMiscException()
@@ -444,7 +498,7 @@ class FindCredentialsTestCase(CredentialsTestCase):
 class ClearCredentialsTestCase(CredentialsTestCase):
     """Test suite for the clear_credentials method."""
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_clear_credentials(self):
         """The credentials are cleared."""
         self.patch(credentials.Keyring, 'delete_credentials',
@@ -453,7 +507,7 @@ class ClearCredentialsTestCase(CredentialsTestCase):
         yield self.obj.clear_credentials()
         self.assertEqual(self._called, ((APP_NAME,), {}))
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_keyring_failure(self):
         """Failures from the keyring are handled."""
         expected_error = SampleMiscException()
@@ -467,7 +521,7 @@ class ClearCredentialsTestCase(CredentialsTestCase):
 class StoreCredentialsTestCase(CredentialsTestCase):
     """Test suite for the store_credentials method."""
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_store_credentials(self):
         """The credentials are stored."""
         self.patch(credentials.Keyring, 'set_credentials',
@@ -476,7 +530,7 @@ class StoreCredentialsTestCase(CredentialsTestCase):
         yield self.obj.store_credentials(TOKEN)
         self.assertEqual(self._called, ((APP_NAME, TOKEN,), {}))
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_keyring_failure(self):
         """Failures from the keyring are handled."""
         expected_error = SampleMiscException()
@@ -492,85 +546,89 @@ class RegisterTestCase(CredentialsTestCase):
 
     operation = 'register'
     login_only = False
+    kwargs = {}
+    inner_class = FakedClientGUI
 
     def setUp(self):
         super(RegisterTestCase, self).setUp()
-        self.ui_kwargs = UI_KWARGS.copy()
-        self.ui_kwargs['login_only'] = self.login_only
+        self.inner_kwargs = UI_KWARGS.copy()
+        self.inner_kwargs['login_only'] = self.login_only
+        self.method_call = getattr(self.obj, self.operation)
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_with_existent_token(self):
         """The operation returns the credentials if already in keyring."""
         self.patch(credentials.Keyring, 'get_credentials',
                    lambda kr, app: defer.succeed(TOKEN))
 
-        yield getattr(self.obj, self.operation)()
+        yield self.method_call(**self.kwargs)
 
         self.assertEqual(self._called, (('success', APP_NAME, TOKEN), {}))
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_without_existent_token(self):
-        """The operation returns the credentials gathered by the GUI."""
+        """The operation returns the credentials gathered by the inner call."""
         self.patch(credentials.Keyring, 'get_credentials',
                    lambda kr, app: defer.succeed(None))
 
-        yield getattr(self.obj, self.operation)()
+        yield self.method_call(**self.kwargs)
 
-        self.assertEqual(self.obj.gui.kwargs, self.ui_kwargs)
+        self.assertEqual(self.obj.inner.kwargs, self.inner_kwargs)
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def test_with_exception_on_credentials(self):
         """The operation calls the error callback if a exception occurs."""
         expected_error = SampleMiscException()
         self.patch(credentials.Keyring, 'get_credentials',
                    lambda kr, app: defer.fail(expected_error))
 
-        yield getattr(self.obj, self.operation)()
+        yield self.method_call(**self.kwargs)
 
         msg = 'Problem while retrieving credentials'
         self.assert_error_cb_called(msg=msg, detailed_error=expected_error)
         self.assertTrue(self.memento.check_exception(SampleMiscException))
 
-    @inlineCallbacks
-    def test_with_exception_on_gui(self):
-        """The operation calls the error callback if a GUI exception occurs."""
+    @defer.inlineCallbacks
+    def test_with_exception_on_inner_call(self, msg=None):
+        """The operation calls the error callback if a exception occurs."""
         self.patch(credentials.Keyring, 'get_credentials',
                    lambda kr, app: defer.succeed(None))
-        err = 'A failing GUI class.'
-        self.obj.ui_class = 'FailingClientGUI'
+        self.obj.ui_class = 'FailingClient'
 
-        yield getattr(self.obj, self.operation)()
+        yield self.method_call(**self.kwargs)
 
-        msg = 'Problem opening the Ubuntu SSO user interface'
+        if msg is None:
+            msg = 'Problem opening the Ubuntu SSO user interface'
         self.assert_error_cb_called(msg=msg,
-                                    detailed_error=SampleMiscException(err))
-        self.assertTrue(self.memento.check_exception(SampleMiscException, err))
+            detailed_error=SampleMiscException(FailingClient.err_msg))
+        self.assertTrue(self.memento.check_exception(SampleMiscException,
+                                                     FailingClient.err_msg))
 
-    @inlineCallbacks
-    def test_connects_gui_signals(self):
-        """GUI callbacks are properly connected."""
+    @defer.inlineCallbacks
+    def test_connects_inner_signals(self):
+        """Inner callbacks are properly connected."""
         self.patch(credentials.Keyring, 'get_credentials',
                    lambda kr, app: defer.succeed(None))
-        yield getattr(self.obj, self.operation)()
+        yield self.method_call(**self.kwargs)
 
-        self.assertEqual(self.obj.gui.login_success_callback,
+        self.assertEqual(self.obj.inner.login_success_callback,
                          self.obj._login_success_cb)
-        self.assertEqual(self.obj.gui.registration_success_callback,
+        self.assertEqual(self.obj.inner.registration_success_callback,
                          self.obj._login_success_cb)
-        self.assertEqual(self.obj.gui.user_cancellation_callback,
+        self.assertEqual(self.obj.inner.user_cancellation_callback,
                          self.obj._auth_denial_cb)
 
-    @inlineCallbacks
-    def test_gui_is_created(self):
-        """The GUI is created and stored."""
+    @defer.inlineCallbacks
+    def test_inner_object_is_created(self):
+        """The inner object is created and stored."""
         self.patch(credentials.Keyring, 'get_credentials',
                    lambda kr, app: defer.succeed(None))
 
-        yield getattr(self.obj, self.operation)()
+        yield self.method_call(**self.kwargs)
 
-        self.assertIsInstance(self.obj.gui, FakedClientGUI)
-        self.assertEqual(self.obj.gui.args, ())
-        self.assertEqual(self.obj.gui.kwargs, self.ui_kwargs)
+        self.assertIsInstance(self.obj.inner, self.inner_class)
+        self.assertEqual(self.obj.inner.args, ())
+        self.assertEqual(self.obj.inner.kwargs, self.inner_kwargs)
 
 
 class LoginTestCase(RegisterTestCase):
@@ -578,3 +636,32 @@ class LoginTestCase(RegisterTestCase):
 
     operation = 'login'
     login_only = True
+
+
+class LoginEmailPasswordTestCase(RegisterTestCase):
+    """Test suite for the login_email_password method."""
+
+    operation = 'login_email_password'
+    login_only = True
+    kwargs = {'email': EMAIL, 'password': PASSWORD}
+    inner_class = FakedSSOLoginRoot
+
+    def setUp(self):
+        super(LoginEmailPasswordTestCase, self).setUp()
+        self.inner_kwargs = {APP_NAME_KEY: APP_NAME, 'email': EMAIL,
+                             'password': PASSWORD,
+                             'result_cb': self.obj._login_success_cb,
+                             'error_cb': self.obj._error_cb,
+                             'not_validated_cb': self.obj._error_cb}
+        self.patch(ubuntu_sso.main, 'SSOLoginRoot', FakedSSOLoginRoot)
+
+    def test_with_exception_on_inner_call(self, msg=None):
+        """The operation calls the error callback if a exception occurs."""
+        self.patch(ubuntu_sso.main, 'SSOLoginRoot', FailingClient)
+        msg = 'Problem logging with email and password.'
+        return super(LoginEmailPasswordTestCase,
+                     self).test_with_exception_on_inner_call(msg=msg)
+
+    def test_connects_inner_signals(self):
+        """Inner callbacks are properly connected."""
+        # there is no inner callbacks for the SSOLoginRoot object
