@@ -43,6 +43,10 @@ COLLECTION_BASE_PATH = "/org/freedesktop/secrets/collection/"
 SAMPLE_CONTENT_TYPE = "text/plain; charset=utf8"
 
 
+class InvalidProperty(Exception):
+    """An exception for when invalid properties are passed in."""
+
+
 class SampleMiscException(Exception):
     """An exception that will be turned into a DBus Exception."""
 
@@ -96,18 +100,6 @@ class ItemMock(dbus.service.Object):
         return True
 
 
-class AltItemMock(ItemMock):
-    """The secret in this item has a content_type."""
-
-    @dbus.service.method(dbus_interface=txsecrets.ITEM_IFACE,
-                         in_signature="o", out_signature="(oayays)")
-    def GetSecret(self, session):
-        """Retrieve the secret for this item."""
-        if self.get_secret_fail:
-            raise SampleMiscException()
-        return (session, "", self.value, SAMPLE_CONTENT_TYPE)
-
-
 class PromptMock(dbus.service.Object):
     """A prompt necessary to complete an operation."""
 
@@ -130,8 +122,8 @@ class PromptMock(dbus.service.Object):
         """The prompt and operation completed."""
 
 
-class CollectionMock(dbus.service.Object):
-    """A collection of items containing secrets."""
+class BaseCollectionMock(dbus.service.Object):
+    """Base collection of items containing secrets."""
     SUPPORTS_MULTIPLE_OBJECT_PATHS = True
     SUPPORTS_MULTIPLE_CONNECTIONS = True
     create_item_prompt = False
@@ -141,16 +133,17 @@ class CollectionMock(dbus.service.Object):
     unlock_prompts = False
     item_mock_class = ItemMock
 
+    item_attrs_property = txsecrets.ITEM_ATTRIBUTES_PROPERTY_OLD
+    item_label_property = txsecrets.ITEM_LABEL_PROPERTY_OLD
+    clxn_label_property = txsecrets.CLXN_LABEL_PROPERTY_OLD
+
     def __init__(self, label, *args, **kwargs):
         """Initialize this instance."""
-        super(CollectionMock, self).__init__(*args, **kwargs)
+        super(BaseCollectionMock, self).__init__(*args, **kwargs)
         self.items = []
         self.label = label
 
-    @dbus.service.method(dbus_interface=txsecrets.COLLECTION_IFACE,
-                         in_signature="a{sv}(oayay)b", out_signature="oo",
-                         byte_arrays=True)
-    def CreateItem(self, properties, secret, replace):
+    def _create_item(self, properties, secret, replace):
         """Create an item with the given attributes, secret and label.
 
         If replace is set, then it replaces an item already present with the
@@ -160,9 +153,9 @@ class CollectionMock(dbus.service.Object):
             raise SampleMiscException()
         if self.locked:
             raise SampleMiscException(ERROR_CREATE_BUT_LOCKED)
-        attributes = properties[txsecrets.ATTRIBUTES_PROPERTY]
-        item_label = properties[txsecrets.LABEL_PROPERTY]
-        session, parameters, value = secret
+        attributes = properties[self.item_attrs_property]
+        item_label = properties[self.item_label_property]
+        value = secret[2]
         item_path = create_object_path(make_coll_path(self.label))
         item = self.dbus_publish(item_path, self.item_mock_class, self,
                                  item_label, attributes, value)
@@ -180,8 +173,22 @@ class CollectionMock(dbus.service.Object):
                          in_signature="ss", out_signature="v")
     def Get(self, interface, propname):
         """The only property implemented is Label."""
-        if interface == txsecrets.COLLECTION_IFACE and propname == "Label":
+        if interface == txsecrets.COLLECTION_IFACE and \
+                propname == self.clxn_label_property:
             return dbus.String(self.label)
+        raise InvalidProperty("Invalid property: {}".format(propname))
+
+
+class CollectionMock(BaseCollectionMock):
+    """Collection of items containing secrets."""
+
+    @dbus.service.method(dbus_interface=txsecrets.COLLECTION_IFACE,
+                         in_signature="a{sv}(oayay)b", out_signature="oo",
+                         byte_arrays=True)
+    def CreateItem(self, properties, secret, replace):
+        """Expose the _create_item method on DBus."""
+        assert len(secret) == 3
+        return self._create_item(properties, secret, replace)
 
 
 class SessionMock(dbus.service.Object):
@@ -203,6 +210,10 @@ class SecretServiceMock(dbus.service.Object):
     create_collection_fail = False
     open_session_fail = False
     dismissed = False
+    collection_mock_class = CollectionMock
+
+    clxn_label_property = txsecrets.CLXN_LABEL_PROPERTY_OLD
+    collections_property = txsecrets.COLLECTIONS_PROPERTY_OLD
 
     def __init__(self, *args, **kwargs):
         """Initialize this instance."""
@@ -228,9 +239,10 @@ class SecretServiceMock(dbus.service.Object):
         """Create a new collection with the specified properties."""
         if self.create_collection_fail:
             raise SampleMiscException()
-        label = str(properties[txsecrets.LABEL_PROPERTY])
+        label = str(properties[self.clxn_label_property])
         coll_path = make_coll_path(label)
-        collection = self.dbus_publish(coll_path, CollectionMock, label)
+        collection = self.dbus_publish(coll_path, self.collection_mock_class,
+                                       label)
         self.collections[label] = collection
 
         if self.create_collection_prompt:
@@ -314,9 +326,61 @@ class SecretServiceMock(dbus.service.Object):
                          in_signature="ss", out_signature="v")
     def Get(self, interface, propname):
         """The only property implemented is Collections."""
-        if interface == txsecrets.SERVICE_IFACE and propname == "Collections":
+        if interface == txsecrets.SERVICE_IFACE and \
+                propname == self.collections_property:
             coll_paths = [make_coll_path(l) for l in self.collections]
             return dbus.Array(coll_paths, signature="o", variant_level=1)
+        raise InvalidProperty("Invalid property: {}".format(propname))
+
+
+class AltItemMock(ItemMock):
+    """The secret in this item has a content_type."""
+
+    @dbus.service.method(dbus_interface=txsecrets.ITEM_IFACE,
+                         in_signature="o", out_signature="(oayays)")
+    def GetSecret2(self, session):
+        """Retrieve the secret for this item."""
+        if self.get_secret_fail:
+            raise SampleMiscException()
+        return (session, "", self.value, SAMPLE_CONTENT_TYPE)
+
+
+class AltCollectionMock(BaseCollectionMock):
+    """The secrets in this collection have a content_type field."""
+
+    item_mock_class = AltItemMock
+
+    item_attrs_property = txsecrets.ITEM_ATTRIBUTES_PROPERTY
+    item_label_property = txsecrets.ITEM_LABEL_PROPERTY
+    clxn_label_property = txsecrets.CLXN_LABEL_PROPERTY
+
+    @dbus.service.method(dbus_interface=txsecrets.COLLECTION_IFACE,
+                         in_signature="a{sv}(oayays)b", out_signature="oo",
+                         byte_arrays=True)
+    def CreateItem(self, properties, secret, replace):
+        """Expose the _create_item method on DBus."""
+        assert len(secret) == 4
+        return self._create_item(properties, secret, replace)
+
+
+class AltSecretServiceMock(SecretServiceMock):
+    """The secrets in this service have a content_type field."""
+
+    collection_mock_class = AltCollectionMock
+
+    clxn_label_property = txsecrets.CLXN_LABEL_PROPERTY
+    collections_property = txsecrets.COLLECTIONS_PROPERTY
+
+    # pylint: disable=W0221
+    @dbus.service.method(dbus_interface=txsecrets.SERVICE_IFACE,
+                         in_signature="a{sv}s", out_signature="oo")
+    def CreateCollection(self, properties, alias):
+        """Create a new collection with the specified properties."""
+        collection, prompt = super(AltSecretServiceMock,
+                                   self).CreateCollection(properties)
+        self.SetAlias(alias, collection)
+        return collection, prompt
+    # pylint: enable=W0221
 
 
 def create_object_path(base):
@@ -328,12 +392,13 @@ def create_object_path(base):
 class BaseTestCase(DBusTestCase):
     """Base class for DBus tests."""
     timeout = 10
+    secret_service_class = SecretServiceMock
 
     def setUp(self):
         super(BaseTestCase, self).setUp()
         self.session_bus = dbus.SessionBus()
         self.mock_service = self.dbus_publish(txsecrets.SECRETS_SERVICE,
-                                              SecretServiceMock)
+                                              self.secret_service_class)
         self.secretservice = txsecrets.SecretService()
 
     def dbus_publish(self, object_path, object_class, *args, **kwargs):
@@ -759,22 +824,6 @@ class ItemTestCase(BaseTestCase):
         value = yield items[0].get_value()
         self.assertEqual(value, sample_secret)
 
-    @inlineCallbacks
-    def test_get_value_four_items_per_secret(self):
-        """The code works fine when the secret dbus struct has 4 items."""
-        yield self.secretservice.open_session()
-        collection_label = "sample_keyring"
-        coll = yield self.create_sample_collection(collection_label)
-        mock_coll = self.mock_service.collections[collection_label]
-        mock_coll.item_mock_class = AltItemMock
-        attr = {"key-type": "Ubuntu SSO credentials"}
-        sample_secret = "secret83!"
-        yield coll.create_item("Cucaracha", attr, sample_secret)
-        items = yield self.secretservice.search_items(attr)
-        self.assertEqual(len(items), 1)
-        value = yield items[0].get_value()
-        self.assertEqual(value, sample_secret)
-
     @inlineCallbacks
     def test_get_value_throws_dbus_error(self):
         """The secret value is not retrieved if DBus fails."""
@@ -853,3 +902,36 @@ class ItemTestCase(BaseTestCase):
         mock_item.delete_fail = True
         yield self.assertFailure(items[0].delete(),
                                  dbus.exceptions.DBusException)
+
+
+class AltItemTestCase(BaseTestCase):
+    """Test the Item class with 4 fields in the secret struct."""
+    secret_service_class = AltSecretServiceMock
+
+    @inlineCallbacks
+    def test_create_item_four_fields_per_secret(self):
+        """The collection creates an item when the dbus struct has 4 fields."""
+        yield self.secretservice.open_session()
+        collection_label = "sample_keyring"
+        yield self.create_sample_collection(collection_label)
+        coll = yield self.secretservice.get_default_collection()
+        mock_collection = self.mock_service.collections[collection_label]
+        attr = {"key-type": "Ubuntu 242 credentials"}
+        sample_secret = "secret!"
+        yield coll.create_item("Cucaracha", attr, sample_secret)
+        self.assertEqual(len(mock_collection.items), 1)
+        self.assertEqual(mock_collection.items[0].value, sample_secret)
+
+    @inlineCallbacks
+    def test_get_value_four_fields_per_secret(self):
+        """The code works fine when the secret dbus struct has 4 fields."""
+        yield self.secretservice.open_session()
+        collection_label = "sample_keyring"
+        coll = yield self.create_sample_collection(collection_label)
+        attr = {"key-type": "Ubuntu SSO credentials"}
+        sample_secret = "secret83!"
+        yield coll.create_item("Cucaracha", attr, sample_secret)
+        items = yield self.secretservice.search_items(attr)
+        self.assertEqual(len(items), 1)
+        value = yield items[0].get_value()
+        self.assertEqual(value, sample_secret)
